﻿local AceAddon = LibStub('AceAddon-3.0')
local AceDB = LibStub('AceDB-3.0')
local AceDBOptions = LibStub('AceDBOptions-3.0')
local AceConfig = LibStub('AceConfig-3.0')
local AceConfigDialog = LibStub('AceConfigDialog-3.0')

local LDB = LibStub("LibDataBroker-1.1")
local LDBI = LibStub("LibDBIcon-1.0")
local LQT = LibStub("LibQTip-1.0")

local ADDON_NAME = 'ToggleMyName'
local ADDON_DB = 'ToggleMyNameDB'

local Addon = AceAddon:NewAddon(ADDON_NAME)
local theVersion = GetAddOnMetadata(ADDON_NAME, "Version")
local theNotes = GetAddOnMetadata(ADDON_NAME, "Notes")

local tmn_tooltip

--------------------------------------------------
-- defaults
--------------------------------------------------
local defaults = {
	profile = {
		minimapPos = 220,
		hide = false,
	},
}

--------------------------------------------------
-- LibDataBroker
--------------------------------------------------
local LDBButton = LDB:NewDataObject(ADDON_NAME, {
	type = "data source",
	icon = "Interface\\AddOns\\ToggleMyName\\Images\\Icon",
	label = ADDON_NAME,
	text = ADDON_NAME
})

--------------------------------------------------
-- functions
--------------------------------------------------
function LDBButton:OnClick(button)
	-- left click for name toggle
	if (button == "LeftButton") then
		ToggleTheName("UnitNameOwn")
		tmn_ShowTooltip()
	end

	-- right click for config
	if (button == "RightButton") then
		InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
	end
end

function LDBButton.OnEnter(self)
	if not (LQT:IsAcquired(ADDON_NAME)) then
		tmn_tooltip = LQT:Acquire(ADDON_NAME, 3, "LEFT", "LEFT", "LEFT")
		tmn_tooltip:SmartAnchorTo(self)
		tmn_tooltip:SetAutoHideDelay(0.25, self)
	end
	tmn_ShowTooltip()
end

function LDBButton.OnLeave(self) end

function tmn_ShowTooltip()
	local line

	tmn_tooltip:Clear()

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "|cffffff00My Name|r")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNameOwn"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNameOwn")

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "|cff00ff00NPC Names|r")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNameNPC"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNameNPC")

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "|cffff0000Friendly Names|r")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNameFriendlyPlayerName"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNameFriendlyPlayerName")

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "|cff0066ccEnemy Names|r")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNameEnemyPlayerName"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNameEnemyPlayerName")

	tmn_tooltip:AddSeparator()

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "Guild Names")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNamePlayerGuild"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNamePlayerGuild")

	-- line = tmn_tooltip:AddLine()
	-- line = tmn_tooltip:SetCell(line, 1, nil)
	-- line = tmn_tooltip:SetCell(line, 2, "Guild Titles")
	-- line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNameGuildTitle"))
	-- tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNameGuildTitle")

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "Titles")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNamePlayerPVPTitle"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNamePlayerPVPTitle")

	line = tmn_tooltip:AddLine()
	line = tmn_tooltip:SetCell(line, 1, nil)
	line = tmn_tooltip:SetCell(line, 2, "Critters and Companions")
	line = tmn_tooltip:SetCell(line, 3, tmn_OnOff("UnitNameNonCombatCreatureName"))
	tmn_tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, "UnitNameNonCombatCreatureName")

	tmn_tooltip:Show()
end

function tmn_OnOff(var)
	if (GetCVar(var) == "1") then return "On" else return "Off" end
end

function Entry_OnMouseUp(frame, var, button)
	if (button == "LeftButton") then
		ToggleTheName(var)
		tmn_ShowTooltip()
	end
end

function ToggleTheName(var)
	SetCVar(var, GetCVar(var) == "1" and 0 or 1)
end

--------------------------------------------------
-- initialize
--------------------------------------------------
function Addon:OnInitialize()

	self.db = AceDB:New(ADDON_DB, defaults, true)
	LDBI:Register(ADDON_NAME, LDBButton, self.db.profile)

	local options = {
		name = ADDON_NAME,
		desc = theNotes,
		order = 0,
		type = 'group',
		args = {
			version = {
				order = 1,
				name = theVersion,
				type = 'description',
			},
			blankdesc1 = {
				order = 2,
				name = ' ',
				type = 'description',
			},
			show_minimap = {
				order = 3,
				name = 'Show Minimap Button',
				desc = 'Toggles the display of the minimap button',
				type = 'toggle',
				get = function()
					return not self.db.profile.hide
				end,
				set = function(_,v)
					self.db.profile.hide = not v
					if self.db.profile.hide then
						LDBI:Hide(ADDON_NAME)
					else
						LDBI:Show(ADDON_NAME)
					end
				end,
				width = 'normal',
			},
			blankdesc3 = {
				order = 4,
				name = ' ',
				type = 'description',
			},
			angle = {
				order = 5,
				name = 'Angle',
				desc = 'Set the angle for the minimap button',
				type = 'range',
				min = 0,
				max = 360,
				step = 1,
				get = function(f)
					return self.db.profile.minimapPos
				end,
				set = function(f, v)
					self.db.profile.minimapPos = v
					if not self.db.profile.hide then
						LDBI:Show(ADDON_NAME)
					end
				end,
			},
		},
	}

	AceConfig:RegisterOptionsTable(ADDON_NAME, options)
	AceConfigDialog:AddToBlizOptions(ADDON_NAME, ADDON_NAME)

end
